-----------------------------------------------------------------------------------------------
-- Client Lua Script for GatherBuddy
-----------------------------------------------------------------------------------------------

require "Window"
require "Unit"
require "GameLib"
require "Apollo"
require "math"

local GatherBuddy = {}

GatherBuddy.colorCodeTradeskill = {
	["Farmer"] = ApolloColor.new("bbddd400"),
	["Survivalist"] = ApolloColor.new("bb37dd00"),
	["Relic Hunter"] = ApolloColor.new("bbdb00dd"),
	["Mining"] = ApolloColor.new("bbdd6a00"),
	["Fishing"] = ApolloColor.new("bb3052dc"),
}
 
function GatherBuddy:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    return o
end

function GatherBuddy:Init()
    Apollo.RegisterAddon(self)
end

function GatherBuddy:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("GatherBuddy.xml")
	self.unitList = {}
	self.wndMain = Apollo.LoadForm(self.xmlDoc, "GatherBuddyForm", nil, self)
	self.wndMain:Show(false)
	self.wndBuddy = self.wndMain:FindChild("Buddy")
	self.wndInternal = self.wndBuddy:FindChild("Internal")
end

function GatherBuddy:GetAsyncLoadStatus()
	if self.xmlDoc:IsLoaded() and g_InterfaceOptionsLoaded then
		self:Initialize()
		return Apollo.AddonLoadStatus.Loaded
	end
	return Apollo.AddonLoadStatus.Loading
end

function GatherBuddy:Initialize()
	Apollo.RegisterEventHandler("UnitCreated", "newUnitCreated", self)
	Apollo.RegisterTimerHandler("OneSecTimer", "OnSecondTimer", self)
	Apollo.RegisterSlashCommand("gathbud", "OnSlashShow", self)
end


function GatherBuddy:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then
		return
	end
	local tSavedData = {}
	tSavedData.tAnchorOffsets = {self.wndBuddy:GetAnchorOffsets()}
	return tSavedData

end

function GatherBuddy:OnRestore(eType, tSavedData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then
		return
	end
	
	if tSavedData.tAnchorOffsets then
		self.wndBuddy:SetAnchorOffsets(unpack(tSavedData.tAnchorOffsets))
	end
end

function GatherBuddy:OnSlashShow()
	self.wndMain:Show(not self.wndBuddy:IsVisible())
end

local function SortTableByDist(a, b)
	return a:GetData().dist < b:GetData().dist
end

function GatherBuddy:OnSecondTimer()
	if self.unitList then
		for idx, v in pairs(self.unitList) do
			local madeUnit = GameLib.GetUnitById(idx)
			if madeUnit then
				v.dist = self:CalculateInfo(madeUnit, v.wnd)
			else v.wnd:Destroy()
				self.unitList[idx] = nil
			end
		end
		self.wndInternal:ArrangeChildrenVert(0, SortTableByDist)
	end
end

function GatherBuddy:CalculateInfo(madeUnit, newInner)
	local harvestable = madeUnit:GetHarvestRequiredTradeskillName()
	local unitPos = madeUnit:GetPosition()
	local unitName = string.sub(madeUnit:GetName(), 0, 30)
	local playerPos = GameLib:GetPlayerUnit():GetPosition()
	local arrowWnd = newInner:FindChild("Window")
	local rotX, rotY = playerPos.z - unitPos.z, unitPos.x - playerPos.x
	local wndRot = math.deg(math.atan2(rotY, rotX))
	local tFacing = GameLib:GetPlayerUnit():GetFacing()
	local pRot = math.deg(math.atan2(tFacing.x, - tFacing.z))
	if math.floor(unitPos.y) < math.floor(playerPos.y) then arrowWnd:SetBGColor(self.colorCodeTradeskill["Mining"])
	elseif math.floor(unitPos.y) > math.floor(playerPos.y) then arrowWnd:SetBGColor(self.colorCodeTradeskill["Farmer"])
	end
	local distToP = math.sqrt((playerPos.x-unitPos.x)^2+(playerPos.y-unitPos.y)^2+(playerPos.z-unitPos.z)^2)
	if type(wndRot) == "number" then
		newInner:FindChild("Window"):SetRotation(wndRot - pRot)
	end
	newInner:SetText(unitName .. " - " .. math.floor(distToP) .. "m")
	if harvestable then
		newInner:SetBGColor(self.colorCodeTradeskill[harvestable])
	else
		newInner:SetBGColor(self.colorCodeTradeskill["Fishing"])
	end
	return distToP
end

function GatherBuddy:newUnitCreated(madeUnit)	
	local harvestable = madeUnit:GetHarvestRequiredTradeskillName()
	if harvestable and self.unitList[madeUnit:GetId()] == nil then
		local newInner = Apollo.LoadForm(self.xmlDoc, "Inner", self.wndInternal, self)
		newInner:Show(true)
		local distToP = self:CalculateInfo(madeUnit, newInner)
		self.unitList[madeUnit:GetId()] = {wnd = newInner; dist = distToP;}
		newInner:SetData(self.unitList[madeUnit:GetId()])
		self.wndInternal:ArrangeChildrenVert(0, SortTableByDist)
	end
end

local GatherBuddyInst = GatherBuddy:new()
GatherBuddyInst:Init()
